package com.landray.kmss.sys.attachment.plugin;

import com.landray.kmss.sys.attachment.model.SysAttMainHistoryConfig;
import java.util.Iterator;
import java.util.ArrayList;
import com.landray.sso.client.oracle.StringUtil;
import com.landray.kmss.framework.plugin.core.config.IExtension;
import java.util.HashMap;
import com.landray.kmss.framework.service.plugin.Plugin;
import java.util.List;
import java.util.Map;

public class HistoryVersionPlugin
{
    private static final String ID = "com.landray.kmss.sys.attachment.history";
    private static Map<String, List<HistoryVersionPluginData>> moduleMap;
    private static Map<String, HistoryVersionPluginData> extensionMap;
    private static Map<String, HistoryVersionPluginData> enabledModels;
    
    static {
        HistoryVersionPlugin.moduleMap = null;
        HistoryVersionPlugin.extensionMap = null;
        HistoryVersionPlugin.enabledModels = null;
    }
    
    private static synchronized void buildExtensionMap() {
        final IExtension[] extensions = Plugin.getExtensions("com.landray.kmss.sys.attachment.history", "*", "historyVersion");
        HistoryVersionPlugin.extensionMap = new HashMap<String, HistoryVersionPluginData>();
        IExtension[] array;
        for (int length = (array = extensions).length, i = 0; i < length; ++i) {
            final IExtension extension = array[i];
            final HistoryVersionPluginData data = new HistoryVersionPluginData();
            data.setModelClassName(Plugin.getParamValueString(extension, "modelClassName"));
            data.setModelName(Plugin.getParamValueString(extension, "modelName"));
            data.setModuleName(Plugin.getParamValueString(extension, "moduleName"));
            HistoryVersionPlugin.extensionMap.put(data.getModelClassName(), data);
        }
    }
    
    private static synchronized void buildModuleMap() {
        HistoryVersionPlugin.moduleMap = new HashMap<String, List<HistoryVersionPluginData>>();
        final Map<String, HistoryVersionPluginData> map = getExtensionMap();
        for (final String key : map.keySet()) {
            final HistoryVersionPluginData data = map.get(key);
            String moduleName = data.getModuleName();
            if (StringUtil.isNull(moduleName)) {
                moduleName = data.getModelName();
            }
            List<HistoryVersionPluginData> list = HistoryVersionPlugin.moduleMap.get(moduleName);
            if (list == null) {
                list = new ArrayList<HistoryVersionPluginData>();
                HistoryVersionPlugin.moduleMap.put(moduleName, list);
            }
            list.add(data);
        }
    }
    
    private static synchronized void buildEnabledModelsMap(String modulesStr) {
        HistoryVersionPlugin.enabledModels = new HashMap<String, HistoryVersionPluginData>();
        if (modulesStr == null) {
            SysAttMainHistoryConfig config = null;
            try {
                config = new SysAttMainHistoryConfig();
            }
            catch (final Exception ex) {
                return;
            }
            modulesStr = config.getAttHistoryConfigEnableModules();
            if (StringUtil.isNull(modulesStr)) {
                return;
            }
        }
        final String[] modules = modulesStr.split(";");
        final Map<String, List<HistoryVersionPluginData>> moduleMap = getModuleMap();
        String[] array;
        for (int length = (array = modules).length, i = 0; i < length; ++i) {
            final String module = array[i];
            final List<HistoryVersionPluginData> list = moduleMap.get(module);
            if (list != null) {
                for (final HistoryVersionPluginData data : list) {
                    HistoryVersionPlugin.enabledModels.put(data.getModelClassName(), data);
                }
            }
        }
    }
    
    public static void resetEnabledModelsMap(final String modulesStr) {
        buildEnabledModelsMap(modulesStr);
    }
    
    public static Map<String, HistoryVersionPluginData> getExtensionMap() {
        if (HistoryVersionPlugin.extensionMap == null) {
            buildExtensionMap();
        }
        return HistoryVersionPlugin.extensionMap;
    }
    
    public static Map<String, List<HistoryVersionPluginData>> getModuleMap() {
        if (HistoryVersionPlugin.moduleMap == null) {
            buildModuleMap();
        }
        return HistoryVersionPlugin.moduleMap;
    }
    
    public static Map<String, HistoryVersionPluginData> getEnabledModules() {
        if (HistoryVersionPlugin.enabledModels == null) {
            buildEnabledModelsMap(null);
        }
        return HistoryVersionPlugin.enabledModels;
    }
    
    public static HistoryVersionPluginData getHistoryPluginData(final String modelClassName) {
        if (HistoryVersionPlugin.extensionMap == null) {
            getExtensionMap();
        }
        return HistoryVersionPlugin.extensionMap.get(modelClassName);
    }
    
    public static boolean isEnableHistory(final String modelClassName) {
        return getEnabledModules().containsKey(modelClassName);
    }
}
